//region Imports
'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
var path = require('path');
var _ = require('lodash');
//endregion

// Babel logging.
function logFileHelpers() {
  var through = require('through2');
  var gutil = require('gulp-util');

  return through.obj(function (file, enc, cb) {
    var src = file.history[0];
    var dest = file.history[1];
    var relPath = function relPath(f) {
      return path.relative('.', f);
    };
    gutil.log('Compiling (Babel):', gutil.colors.red(relPath(src) + ' -> ' + relPath(dest)));
    //gutil.log(file.babel.usedHelpers)
    cb(null, file);
  });
}

exports['default'] = function (gulp, opts) {

  _.defaults(opts, {
    srcGlob: '',
    shouldCompileTests: false,
    destDir: '',
    testsGlob: '',
    babel: {
      stage: 0,
      optional: ['runtime']
    }
  });

  gulp.task('lib:compile', function () {
    var gulpif = require('gulp-if');
    var changed = require('gulp-changed');
    var ignore = require('gulp-ignore');
    var babel = require('gulp-babel');
    var sourcemaps = require('gulp-sourcemaps');

    return gulp.src(opts.srcGlob).pipe(gulpif(gulp.isWatching, changed(opts.destDir))).pipe(gulpif(!opts.shouldCompileTests, ignore.exclude(opts.testsGlob))).pipe(sourcemaps.init()).pipe(babel(opts.babel))
    // Source maps work with IntelliJ debugger using Node.js 0.12.7.
    .pipe(sourcemaps.write('.', {
      includeContent: false,
      sourceRoot: function sourceRoot(file) {
        return path.relative(file.path, __dirname);
      }
    })).pipe(gulp.dest(opts.destDir)).pipe(logFileHelpers());
  });

  gulp.task('lib:clean', function () {
    var del = require('del');
    var gutil = require('gulp-util');

    gutil.log('Cleaning:', gutil.colors.red(opts.destDir));
    return del(opts.destGlob);
  });

  gulp.task('lib:watch', gulp.series('lib:clean', 'lib:compile', function watcher() {
    var gutil = require('gulp-util');

    // Used by `compile` task to check whether we should only compile changed files.
    gulp.isWatching = true;

    gulp.watch(opts.srcGlob, gulp.series('lib:compile')).on('change', function (evt) {
      gutil.log('---');
      gutil.log('File changed:', gutil.colors.red(evt.path));
    });
  }));
};

module.exports = exports['default'];