//region Imports
const _ = require('lodash')
const cwd = require('cwd')
const path = require('path')
//endregion

export default (gulp, opts) => {

  _.defaults(opts, {
    srcGlob: '',
    testsGlob: '',
    mocha: {
      r: ['babel/register', 'babelator/defaults/mocha'],
      //reporter: 'nyan',
    },
    testsDestGlob: '',
  })

  // TODO(vjpr): Not teted.
  gulp.task('karma:start', (done) => {
    const karma = require('karma')
    new karma.Server({
      configFile: cwd('karma.conf.js'),
      singleRun: true,
    }, done).start()
  })

  function handleError(e) {
    console.log(e.toString())
    if (gulp.isWatching) {
      this.emit('end')
      return
    }
    process.exit(1)
  }

  gulp.task('mocha:src', () => {
    const mocha = require('gulp-spawn-mocha')

    return gulp.src(opts.testsGlob, {read: false})
      .pipe(mocha(opts.mocha)).on('error', handleError)
  })

  // Faster: Run tests on transpiled files so no need for slow babel/require.
  gulp.task('mocha:lib:run', function() {
    const mocha = require('gulp-spawn-mocha')

    if (!opts.shouldCompileTests) throw 'shouldCompileTests must be enabled for `mocha:lib:run` task'

    // TODO(vjpr): Should we clone first?
    opts.mocha.r = _.without(opts.mocha.r, 'babel/register')
    return gulp.src(opts.testsDestGlob, {read: false})
      .pipe(mocha(opts.mocha)).on('error', handleError)

  })

  gulp.task('mocha:lib', gulp.series('lib:compile', 'mocha:lib:run'))

  gulp.task('mocha:set-watching', (done) => {
    gulp.isWatching = true
    done()
  })

  gulp.task('test:watch', gulp.series('mocha:set-watching', 'mocha:lib', function watcher() {
    const gutil = require('gulp-util')

    return gulp.watch(opts.srcGlob, gulp.series('mocha:lib')).on('change', (evt) => {
      gutil.log('---')
      gutil.log('File changed:', gutil.colors.red(evt.path))
    })
  }))

  gulp.task('test:bootstrap:pjson', (done) => {
    const modifyPackageJson = require('../util/modifyPackageJson')

    modifyPackageJson((json) => {
      //_.set(json, 'scripts.karma', './node_modules/karma/bin/karma start')
      _.set(json, 'scripts.test', 'npm run gulp test')
      return json
    }, done)
  })

  gulp.task('test:bootstrap:copy-files', () => {
    const conflict = require('gulp-conflict')

    // TODO(vjpr): This is hacky. We need to src the precompiled version.
    // Move to templates dir or ignore when compiling.
    return gulp.src(path.resolve(__dirname, '../../templates/test/**/*.*'))
      .pipe(conflict('.'))
      .pipe(gulp.dest('.'))
  })

  gulp.task('test:bootstrap', gulp.series('test:bootstrap:copy-files', 'test:bootstrap:pjson'))

}
